@@ -47,7 +47,6 @@ _ASYNC_READER = _symbol('ASYNC_READER')
 
 This state indicates that the transaction is a read-only and is
 safe to use on an asynchronously updated slave database.
-
 """
 
 _READER = _symbol('READER')
@@ -56,7 +55,6 @@ _READER = _symbol('READER')
 This state indicates that the transaction is a read-only and is
 only safe to use on a synchronously updated slave database; otherwise
 the master database should be used.
-
 """
 
 
@@ -65,18 +63,16 @@ _WRITER = _symbol('WRITER')
 
 This state indicates that the transaction writes data and
 should be directed at the master database.
-
 """
 
 
-class _Default(object):
+class _Default:
     """Mark a value as a default value.
 
     A value in the local configuration dictionary wrapped with
     _Default() will not take precedence over a value that is specified
     in cfg.CONF.   Values that are set after the fact using configure()
     will supersede those in cfg.CONF.
-
     """
 
     __slots__ = 'value',
@@ -98,9 +94,9 @@ class _Default(object):
             return value
 
     @classmethod
-    def resolve_w_conf(cls, value, conf_namespace, key):
+    def resolve_w_conf(cls, value, conf, key):
         if isinstance(value, _Default):
-            v = getattr(conf_namespace, key, value.value)
+            v = getattr(conf.database, key, value.value)
             if v is cls._notset:
                 return None
             else:
@@ -110,32 +106,49 @@ class _Default(object):
 
     @classmethod
     def is_set(cls, value):
-        return not isinstance(value, _Default) or \
-            value.value is not cls._notset
+        if not isinstance(value, _Default):
+            return True
+
+        return value.value is not cls._notset
 
     @classmethod
-    def is_set_w_conf(cls, value, conf_namespace, key):
-        return not isinstance(value, _Default) or \
-            value.value is not cls._notset or \
-            hasattr(conf_namespace, key)
+    def is_set_w_conf(cls, value, conf, key):
+        if hasattr(conf.database, key):
+            # If the option is set via configuration then we should always
+            # respect it...unless that option is deprecated and hasn't been set
+            # by user, in which case we should ignore it.
+
+            # oslo.config doesn't provide a public API to retrieve the opt
+            # itself, as opposed to the value of the opt :(
+            opt = conf.database._group._opts[key]['opt']
+            # ditto for the group
+            group = conf.database._group
+            if (
+                opt.deprecated_for_removal and
+                conf.get_location(key, group=group.name).location ==
+                cfg.Locations.opt_default
+            ):
+                return False
+
+            return True
+
+        return cls.is_set(value)
 
 
 class AlreadyStartedError(TypeError):
     """Raises when a factory is being asked to initialize a second time.
 
     Subclasses :class:`.TypeError` for legacy support.
-
     """
 
 
-class _TransactionFactory(object):
+class _TransactionFactory:
     """A factory for :class:`._TransactionContext` objects.
 
     By default, there is just one of these, set up
     based on CONF, however instance-level :class:`._TransactionFactory`
     objects can be made, as is the case with the
     :class:`._TestTransactionFactory` subclass used by the oslo.db test suite.
-
     """
     def __init__(self):
         self._url_cfg = {
@@ -146,7 +159,6 @@ class _TransactionFactory(object):
             'sqlite_fk': _Default(False),
             'mysql_sql_mode': _Default('TRADITIONAL'),
             'mysql_wsrep_sync_wait': _Default(),
-            'mysql_enable_ndb': _Default(False),
             'connection_recycle_time': _Default(3600),
             'connection_debug': _Default(0),
             'max_pool_size': _Default(),
@@ -164,7 +176,7 @@ class _TransactionFactory(object):
         }
         self._maker_cfg = {
             'expire_on_commit': _Default(False),
-            '__autocommit': False,
+            '__autocommit': _Default(False),
         }
         self._transaction_ctx_cfg = {
             'rollback_reader_sessions': False,
@@ -183,7 +195,9 @@ class _TransactionFactory(object):
                 'db_max_retries', 'db_inc_retry_interval',
                 'use_db_reconnect',
                 'db_retry_interval',
-                'db_max_retry_interval', 'backend', 'use_tpool'])
+                'db_max_retry_interval', 'backend', 'use_tpool'
+            ]
+        )
 
         self._started = False
         self._legacy_facade = None
@@ -211,92 +225,69 @@ class _TransactionFactory(object):
         :data:`oslo_db.options.database_opts` configurational defaults to it.
 
         :param connection: database URL
-
         :param slave_connection: database URL
-
         :param sqlite_fk: whether to enable SQLite foreign key pragma; default
-         False
-
+            False
         :param mysql_sql_mode: MySQL SQL mode, defaults to TRADITIONAL
-
         :param mysql_wsrep_sync_wait: MySQL wsrep_sync_wait, defaults to None,
-         which indicates no setting will be passed
-
-        :param mysql_enable_ndb: enable MySQL Cluster (NDB) support
-
+            which indicates no setting will be passed
         :param connection_recycle_time: connection pool recycle time,
-         defaults to 3600. Note the connection does not actually have to
-         be "idle" to be recycled.
-
+            defaults to 3600. Note the connection does not actually have to be
+            "idle" to be recycled.
         :param connection_debug: engine logging level, defaults to 0. set to
-         50 for INFO, 100 for DEBUG.
-
+            50 for INFO, 100 for DEBUG.
         :param connection_parameters: additional parameters to append onto the
-         database URL query string, pass as "param1=value1&param2=value2&..."
-
+            database URL query string, pass as
+            "param1=value1&param2=value2&..."
         :param max_pool_size: max size of connection pool, uses CONF for
-         default
-
+            default
         :param max_overflow: max overflow for connection pool, uses CONF for
-         default
-
+            default
         :param sqlite_synchronous: disable SQLite SYNCHRONOUS pragma if False;
-         defaults to True
-
+            defaults to True
         :param connection_trace: enable tracing comments in logging
-
         :param max_retries: max retries to connect, defaults to !0
-
         :param retry_interval: time in seconds between retries, defaults to 10
-
         :param thread_checkin: add sleep(0) on connection checkin to allow
-         greenlet yields, defaults to True
-
+            greenlet yields, defaults to True
         :param json_serializer: JSON serializer for PostgreSQL connections
-
         :param json_deserializer: JSON deserializer for PostgreSQL connections
-
         :param logging_name: logging name for engine
-
         :param expire_on_commit: sets expire_on_commit for SQLAlchemy
-         sessionmaker; defaults to False
-
+            sessionmaker; defaults to False
         :param rollback_reader_sessions: if True, a :class:`.Session` object
-         will have its :meth:`.Session.rollback` method invoked at the end
-         of a ``@reader`` block, actively rolling back the transaction and
-         expiring the objects within, before the :class:`.Session` moves
-         on to be closed, which has the effect of releasing connection
-         resources back to the connection pool and detaching all objects.
-         If False, the :class:`.Session` is
-         not affected at the end of a ``@reader`` block; the underlying
-         connection referred to by this :class:`.Session` will still
-         be released in the enclosing context via the :meth:`.Session.close`
-         method, which still ensures that the DBAPI connection is rolled
-         back, however the objects associated with the :class:`.Session`
-         retain their database-persisted contents after they are detached.
+            will have its :meth:`.Session.rollback` method invoked at the end
+            of a ``@reader`` block, actively rolling back the transaction and
+            expiring the objects within, before the :class:`.Session` moves on
+            to be closed, which has the effect of releasing connection
+            resources back to the connection pool and detaching all objects.
+            If False, the :class:`.Session` is not affected at the end of a
+            ``@reader`` block; the underlying connection referred to by this
+            :class:`.Session` will still be released in the enclosing context
+            via the :meth:`.Session.close` method, which still ensures that the
+            DBAPI connection is rolled back, however the objects associated
+            with the :class:`.Session` retain their database-persisted contents
+            after they are detached.
 
-         .. seealso::
-
-            http://docs.sqlalchemy.org/en/rel_0_9/glossary.html#term-released\
-            SQLAlchemy documentation on what "releasing resources" means.
+            .. seealso::
 
+                http://docs.sqlalchemy.org/en/rel_0_9/glossary.html#term-released\
+                SQLAlchemy documentation on what "releasing resources" means.
         :param synchronous_reader: whether or not to assume a "reader" context
-         needs to guarantee it can read data committed by a "writer" assuming
-         replication lag is present; defaults to True.  When False, a
-         @reader context works the same as @async_reader and will select
-         the "slave" database if present.
-
+            needs to guarantee it can read data committed by a "writer"
+            assuming replication lag is present; defaults to True.  When False,
+            a @reader context works the same as @async_reader and will select
+            the "slave" database if present.
         :param flush_on_subtransaction: if True, a :class:`.Session` object
-         will have its :meth:`.Session.flush` method invoked whenever a context
-         manager or decorator that is not itself the originator of the top-
-         level or savepoint :class:`.Session` transaction exits - in this way
-         it behaves like a "subtransaction" from a :class:`.Session`
-         perspective.
+            will have its :meth:`.Session.flush` method invoked whenever a
+            context manager or decorator that is not itself the originator of
+            the top- level or savepoint :class:`.Session` transaction exits -
+            in this way it behaves like a "subtransaction" from a
+            :class:`.Session` perspective.
 
         .. seealso::
 
             :meth:`._TransactionFactory.configure`
-
         """
         self._configure(True, kw)
 
@@ -315,29 +306,22 @@ class _TransactionFactory(object):
         .. seealso::
 
             :meth:`._TransactionFactory.configure_defaults`
-
         """
         self._configure(False, kw)
 
     def _configure(self, as_defaults, kw):
-        if 'mysql_enable_ndb' in kw:
-            debtcollector.deprecate(
-                (
-                    'Support for the MySQL NDB Cluster storage engine has '
-                    'been deprecated and will be removed in a future release.'
-                ),
-                version='12.1.0',
-            )
-
         if self._started:
             raise AlreadyStartedError(
-                "this TransactionFactory is already started")
+                "this TransactionFactory is already started"
+            )
+
         not_supported = []
         for k, v in kw.items():
             for dict_ in (
                     self._url_cfg, self._engine_cfg,
                     self._maker_cfg, self._ignored_cfg,
-                    self._facade_cfg, self._transaction_ctx_cfg):
+                    self._facade_cfg, self._transaction_ctx_cfg,
+            ):
                 if k in dict_:
                     dict_[k] = _Default(v) if as_defaults else v
                     break
@@ -361,7 +345,6 @@ class _TransactionFactory(object):
         as this factory, however will not share the same transaction context;
         the legacy facade continues to work the old way of returning
         a new Session each time get_session() is called.
-
         """
         if not self._legacy_facade:
             self._legacy_facade = LegacyEngineFacade(None, _factory=self)
@@ -374,7 +357,6 @@ class _TransactionFactory(object):
         """Return the writer engine for this factory.
 
         Implies start.
-
         """
         if not self._started:
             self._start()
@@ -384,7 +366,6 @@ class _TransactionFactory(object):
         """Return the reader engine for this factory.
 
         Implies start.
-
         """
         if not self._started:
             self._start()
@@ -394,7 +375,6 @@ class _TransactionFactory(object):
         """Return the writer sessionmaker for this factory.
 
         Implies start.
-
         """
         if not self._started:
             self._start()
@@ -404,7 +384,6 @@ class _TransactionFactory(object):
         """Return the reader sessionmaker for this factory.
 
         Implies start.
-
         """
         if not self._started:
             self._start()
@@ -448,17 +427,17 @@ class _TransactionFactory(object):
 
     def _args_for_conf(self, default_cfg, conf):
         if conf is None:
-            return dict(
-                (key, _Default.resolve(value))
+            return {
+                key: _Default.resolve(value)
                 for key, value in default_cfg.items()
                 if _Default.is_set(value)
-            )
+            }
         else:
-            return dict(
-                (key, _Default.resolve_w_conf(value, conf.database, key))
+            return {
+                key: _Default.resolve_w_conf(value, conf, key)
                 for key, value in default_cfg.items()
-                if _Default.is_set_w_conf(value, conf.database, key)
-            )
+                if _Default.is_set_w_conf(value, conf, key)
+            }
 
     def _url_args_for_conf(self, conf):
         return self._args_for_conf(self._url_cfg, conf)
@@ -468,7 +447,8 @@ class _TransactionFactory(object):
 
     def _maker_args_for_conf(self, conf):
         maker_args = self._args_for_conf(self._maker_cfg, conf)
-        maker_args['autocommit'] = maker_args.pop('__autocommit')
+        if '__autocommit' in maker_args:
+            maker_args['autocommit'] = maker_args.pop('__autocommit')
         return maker_args
 
     def dispose_pool(self):
@@ -534,7 +514,8 @@ class _TransactionFactory(object):
             self._started = True
 
     def _setup_for_connection(
-            self, sql_connection, engine_kwargs, maker_kwargs):
+        self, sql_connection, engine_kwargs, maker_kwargs,
+    ):
         if sql_connection is None:
             raise exception.CantStartEngineError(
                 "No sql_connection parameter is established")
@@ -555,7 +536,7 @@ class _TestTransactionFactory(_TransactionFactory):
     Note that while this is used by oslo.db's own tests of
     the enginefacade system, it is also exported for use by
     the test suites of other projects, first as an element of the
-    oslo_db.sqlalchemy.test_base module, and secondly may be used by
+    oslo_db.sqlalchemy.test_fixtures module, and secondly may be used by
     external test suites directly.
 
     Includes a feature to inject itself temporarily as the factory
@@ -596,13 +577,11 @@ class _TransactionContext(object):
         """Construct a new :class:`.TransactionContext`.
 
         :param factory: the :class:`.TransactionFactory` which will
-         serve as a source of connectivity.
-
+            serve as a source of connectivity.
         :param global_factory: the "global" factory which will be used
-         by the global ``_context_manager`` for new ``_TransactionContext``
-         objects created under this one.  When left as None the actual
-         "global" factory is used.
-
+            by the global ``_context_manager`` for new ``_TransactionContext``
+            objects created under this one.  When left as None the actual
+            "global" factory is used.
         """
         self.factory = factory
         self.global_factory = global_factory
@@ -1239,6 +1218,7 @@ class LegacyEngineFacade(object):
     :type sqlite_fk: bool
 
     :param autocommit: use autocommit mode for created Session instances
+                       (only supported with SQLAlchemy < 2.0)
     :type autocommit: bool
 
     :param expire_on_commit: expire session objects on commit
@@ -1251,10 +1231,6 @@ class LegacyEngineFacade(object):
     :keyword mysql_wsrep_sync_wait: value of wsrep_sync_wait for Galera
                              (defaults to None, which indicates no setting
                              will be passed)
-    :keyword mysql_enable_ndb: If True, transparently enables support for
-                               handling MySQL Cluster (NDB).
-                               (defaults to False)
-                               **DEPRECATED**
     :keyword connection_recycle_time: Time period for connections to be
                             recycled upon checkout (defaults to 3600)
     :keyword connection_debug: verbosity of SQL debugging information.
@@ -1283,7 +1259,8 @@ class LegacyEngineFacade(object):
     """
     def __init__(self, sql_connection, slave_connection=None,
                  sqlite_fk=False, autocommit=False,
-                 expire_on_commit=False, _conf=None, _factory=None, **kwargs):
+                 expire_on_commit=False, _conf=None,
+                 _factory=None, **kwargs):
         warnings.warn(
             "EngineFacade is deprecated; please use "
             "oslo_db.sqlalchemy.enginefacade",
@@ -1292,7 +1269,7 @@ class LegacyEngineFacade(object):
 
         if autocommit is True:
             warnings.warn(
-                'autocommit support will be removed in SQLAlchemy 2.0 and '
+                'autocommit support has been removed in SQLAlchemy 2.0 and '
                 'should not be relied on; please rework your code to remove '
                 'reliance on this feature',
                 warning.OsloDBDeprecationWarning,
@@ -1381,6 +1358,7 @@ class LegacyEngineFacade(object):
         :type sqlite_fk: bool
 
         :param autocommit: use autocommit mode for created Session instances
+                           (only supported with SQLAlchemy < 2.0)
         :type autocommit: bool
 
         :param expire_on_commit: expire session objects on commit
