@@ -20,7 +20,7 @@ from sqlalchemy import event
 from sqlalchemy import exc as sqla_exc
 
 from oslo_db import exception
-
+from oslo_db.sqlalchemy import compat
 
 LOG = logging.getLogger(__name__)
 
@@ -44,7 +44,7 @@ def filters(dbname, exception_type, regex):
     """
     def _receive(fn):
         _registry[dbname][exception_type].extend(
-            (fn, re.compile(reg))
+            (fn, re.compile(reg, re.DOTALL))
             for reg in
             ((regex,) if not isinstance(regex, tuple) else regex)
         )
@@ -67,29 +67,25 @@ def filters(dbname, exception_type, regex):
          r"^.*\b1213\b.*Deadlock: wsrep aborted.*")
 @filters("postgresql", sqla_exc.OperationalError, r"^.*deadlock detected.*")
 @filters("postgresql", sqla_exc.DBAPIError, r"^.*deadlock detected.*")
-@filters("ibm_db_sa", sqla_exc.DBAPIError, r"^.*SQL0911N.*")
 def _deadlock_error(operational_error, match, engine_name, is_disconnect):
     """Filter for MySQL or Postgresql deadlock error.
 
     NOTE(comstud): In current versions of DB backends, Deadlock violation
     messages follow the structure:
 
-    mysql+mysqldb:
-    (OperationalError) (1213, 'Deadlock found when trying to get lock; try '
-                         'restarting transaction') <query_str> <query_args>
+    mysql+mysqldb::
 
-    mysql+mysqlconnector:
-    (InternalError) 1213 (40001): Deadlock found when trying to get lock; try
-                         restarting transaction
+        (OperationalError) (1213, 'Deadlock found when trying to get lock; '
+            'try restarting transaction') <query_str> <query_args>
 
-    postgresql:
-    (TransactionRollbackError) deadlock detected <deadlock_details>
+    mysql+mysqlconnector::
 
+        (InternalError) 1213 (40001): Deadlock found when trying to get lock;
+            try restarting transaction
 
-    ibm_db_sa:
-    SQL0911N The current transaction has been rolled back because of a
-    deadlock or timeout <deadlock details>
+    postgresql::
 
+        (TransactionRollbackError) deadlock detected <deadlock_details>
     """
     raise exception.DBDeadlock(operational_error)
 
@@ -120,6 +116,12 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
     N columns - (IntegrityError) duplicate key value violates unique
                constraint "name_of_our_constraint"
 
+    mysql since 8.0.19:
+    1 column - (IntegrityError) (1062, "Duplicate entry 'value_of_c1' for key
+               'table_name.c1'")
+    N columns - (IntegrityError) (1062, "Duplicate entry 'values joined
+               with -' for key 'table_name.name_of_our_constraint'")
+
     mysql+mysqldb:
     1 column - (IntegrityError) (1062, "Duplicate entry 'value_of_c1' for key
                'c1'")
@@ -131,9 +133,6 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
                key 'c1'
     N columns - (IntegrityError) 1062 (23000): Duplicate entry 'values
                joined with -' for key 'name_of_our_constraint'
-
-
-
     """
 
     columns = match.group('columns')
@@ -145,6 +144,9 @@ def _default_dupe_key_error(integrity_error, match, engine_name,
     if not columns.startswith(uniqbase):
         if engine_name == "postgresql":
             columns = [columns[columns.index("_") + 1:columns.rindex("_")]]
+        elif (engine_name == "mysql") and \
+             (uniqbase in str(columns.split("0")[:1])):
+            columns = columns.split("0")[1:]
         else:
             columns = [columns]
     else:
@@ -175,7 +177,6 @@ def _sqlite_dupe_key_error(integrity_error, match, engine_name, is_disconnect):
 
     sqlite since 3.8.2:
     (IntegrityError) PRIMARY KEY must be unique
-
     """
     columns = []
     # NOTE(ochuprykov): We can get here by last filter in which there are no
@@ -312,24 +313,6 @@ def _check_database_non_existing(
     raise exception.DBNonExistentDatabase(database, error)
 
 
-@filters("ibm_db_sa", sqla_exc.IntegrityError, r"^.*SQL0803N.*$")
-def _db2_dupe_key_error(integrity_error, match, engine_name, is_disconnect):
-    """Filter for DB2 duplicate key errors.
-
-    N columns - (IntegrityError) SQL0803N  One or more values in the INSERT
-                statement, UPDATE statement, or foreign key update caused by a
-                DELETE statement are not valid because the primary key, unique
-                constraint or unique index identified by "2" constrains table
-                "NOVA.KEY_PAIRS" from having duplicate values for the index
-                key.
-
-    """
-
-    # NOTE(mriedem): The ibm_db_sa integrity error message doesn't provide the
-    # columns so we have to omit that from the DBDuplicateEntry error.
-    raise exception.DBDuplicateEntry([], integrity_error)
-
-
 @filters("mysql", sqla_exc.DBAPIError, r".*\b1146\b")
 def _raise_mysql_table_doesnt_exist_asis(
         error, match, engine_name, is_disconnect):
@@ -391,10 +374,10 @@ def _raise_operational_errors_directly_filter(operational_error,
 @filters("mysql", sqla_exc.InternalError, r".*\(.*(?:1927)")  # noqa
 @filters("mysql", sqla_exc.InternalError, r".*Packet sequence number wrong")  # noqa
 @filters("postgresql", sqla_exc.OperationalError, r".*could not connect to server")  # noqa
-@filters("ibm_db_sa", sqla_exc.OperationalError, r".*(?:30081)")
 def _is_db_connection_error(operational_error, match, engine_name,
                             is_disconnect):
     """Detect the exception as indicating a recoverable error on connect."""
+
     raise exception.DBConnectionError(operational_error)
 
 
@@ -441,44 +424,63 @@ def handler(context):
     more specific exception class are attempted first.
 
     """
-    def _dialect_registries(engine):
-        if engine.dialect.name in _registry:
-            yield _registry[engine.dialect.name]
+    def _dialect_registries(dialect):
+        if dialect.name in _registry:
+            yield _registry[dialect.name]
         if '*' in _registry:
             yield _registry['*']
 
-    for per_dialect in _dialect_registries(context.engine):
-        for exc in (
-                context.sqlalchemy_exception,
-                context.original_exception):
-            for super_ in exc.__class__.__mro__:
-                if super_ in per_dialect:
-                    regexp_reg = per_dialect[super_]
-                    for fn, regexp in regexp_reg:
-                        match = regexp.match(exc.args[0])
-                        if match:
-                            try:
-                                fn(
-                                    exc,
-                                    match,
-                                    context.engine.dialect.name,
-                                    context.is_disconnect)
-                            except exception.DBError as dbe:
-                                if (
-                                    context.connection is not None and
-                                    not context.connection.closed and
-                                    not context.connection.invalidated and
-                                    ROLLBACK_CAUSE_KEY
-                                    in context.connection.info
-                                ):
-                                    dbe.cause = \
-                                        context.connection.info.pop(
-                                            ROLLBACK_CAUSE_KEY)
+    # do not reraise for our own exceptions
+    if isinstance(context.original_exception, exception.DBError):
+        return
 
-                                if isinstance(
-                                        dbe, exception.DBConnectionError):
-                                    context.is_disconnect = True
-                                return dbe
+    dialect = compat.dialect_from_exception_context(context)
+    for per_dialect in _dialect_registries(dialect):
+        for exc in (context.sqlalchemy_exception, context.original_exception):
+            for super_ in exc.__class__.__mro__:
+                if super_ not in per_dialect:
+                    continue
+
+                regexp_reg = per_dialect[super_]
+                for fn, regexp in regexp_reg:
+                    match = regexp.match(exc.args[0])
+                    if not match:
+                        continue
+
+                    try:
+                        fn(
+                            exc,
+                            match,
+                            dialect.name,
+                            context.is_disconnect,
+                        )
+                    except exception.DBError as dbe:
+                        if (
+                            context.connection is not None and
+                            not context.connection.closed and
+                            not context.connection.invalidated and
+                            ROLLBACK_CAUSE_KEY in context.connection.info
+                        ):
+                            dbe.cause = context.connection.info.pop(
+                                ROLLBACK_CAUSE_KEY,
+                            )
+
+                        if isinstance(dbe, exception.DBConnectionError):
+                            context.is_disconnect = True
+
+                            # new in 2.0.5
+                            if (
+                                hasattr(context, "is_pre_ping") and
+                                context.is_pre_ping
+                            ):
+                                # if this is a pre-ping, need to
+                                # integrate with the built
+                                # in pre-ping handler that doesnt know
+                                # about DBConnectionError, just needs
+                                # the updated status
+                                return None
+
+                        return dbe
 
 
 def register_engine(engine):
